package com.uas.ps.sync.core.intercept;

import com.uas.ps.sync.core.MessageProducer;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.AfterReturning;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.springframework.kafka.core.KafkaTemplate;

/**
 * 监听 CRUD 操作
 *
 * @author sunyj
 * @since 2018/1/24 11:42
 */
@Aspect
public class CRUDIntercept {

    private MessageProducer producer;

    public CRUDIntercept(KafkaTemplate kafkaTemplate) {
        producer = new MessageProducer(kafkaTemplate);
    }

    @Pointcut("target(org.springframework.data.repository.CrudRepository)")
    public void crud() {
    }

    @AfterReturning(pointcut = "(execution(* save(*)) || execution(* saveAndFlush(*))) && crud()", returning = "result")
    public void afterSaved(Object result) {
        producer.addSaveMessage(result);
    }

    @Around("execution(* delete*(..)) && crud()")
    public Object aroundDeleting(ProceedingJoinPoint proceedingJoinPoint) throws Throwable {
        producer.addDeleteMessage(proceedingJoinPoint);
        return proceedingJoinPoint.proceed();
    }

}
