package com.uas.ps.sync.core.util;

import org.springframework.aop.framework.Advised;
import org.springframework.aop.support.AopUtils;

import java.lang.reflect.Field;

/**
 * @author sunyj
 * @since 2018/1/26 9:31
 */
public class ReflectUtils {

    /**
     * 获取指定类的指定字段
     *
     * @param field 指定字段
     * @param clazz 指定类
     * @return 指定字段
     * @throws IllegalArgumentException
     */
    public static Field getDeclaredField(String field, Class<?> clazz) throws IllegalArgumentException {
        try {
            return clazz.getDeclaredField(field);
        } catch (NoSuchFieldException e) {
            throw new IllegalArgumentException(clazz + "中不存在字段：" + field);
        }
    }

    /**
     * 利用反射获取指定对象的指定字段的值
     *
     * @param field 指定字段
     * @param k     指定对象
     * @return 指定字段的值
     */
    public static <K> Object getValue(Field field, K k) throws IllegalStateException {
        try {
            Object value;
            if (!field.isAccessible()) {
                field.setAccessible(true);
                value = field.get(k);
                field.setAccessible(false);
            } else {
                value = field.get(k);
            }
            return value;
        } catch (SecurityException | IllegalArgumentException | IllegalAccessException e) {
            throw new IllegalStateException("通过反射取值失败", e);
        }
    }

    /**
     * 利用反射获取指定对象的指定字段的值
     *
     * @param field 指定字段
     * @param k     指定对象
     * @return 指定字段的值
     */
    public static <K> Object getValue(String field, K k) throws IllegalStateException {
        Field declaredField = getDeclaredField(field, k.getClass());
        return getValue(declaredField, k);
    }

    public static <T> T getTargetObject(Object proxy) throws Exception {
        if (AopUtils.isJdkDynamicProxy(proxy)) {
            return (T) getTargetObject(((Advised) proxy).getTargetSource().getTarget());
        }
        return (T) proxy; // expected to be cglib proxy then, which is simply a specialized class
    }
}
