package com.uas.ps.sync.converter;

import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.core.JsonParser;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.uas.ps.core.util.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.persistence.AttributeConverter;
import java.io.IOException;

/**
 * JSON 转换
 *
 * @author sunyj
 * @since 2018/1/15 15:10
 */
public class JpaJsonDocumentsConverter implements
        AttributeConverter<JSONObject, String> {

    // ObjectMapper is thread safe
    private final static ObjectMapper objectMapper = new ObjectMapper();

    private Logger logger = LoggerFactory.getLogger(getClass());

    {
        objectMapper.configure(JsonParser.Feature.ALLOW_UNQUOTED_CONTROL_CHARS, true);
        objectMapper.configure(DeserializationFeature.ACCEPT_EMPTY_ARRAY_AS_NULL_OBJECT, true);
    }

    @Override
    public String convertToDatabaseColumn(JSONObject meta) {
        String jsonString = "";
        try {
            logger.debug("Start convertToDatabaseColumn");

            // convert list of POJO to json
            jsonString = objectMapper.writeValueAsString(meta);
            logger.debug("convertToDatabaseColumn" + jsonString);

        } catch (JsonProcessingException ex) {
            logger.error(ex.getMessage());
        }
        // 在 convertToEntityAttribute 中，对反斜杠进行了转换，因此这里进行还原
        jsonString = jsonString.replace("\\\\", "\\");
        return jsonString;
    }

    @Override
    public JSONObject convertToEntityAttribute(String dbData) {
        JSONObject jsonObject = new JSONObject();
        try {
            logger.debug("Start convertToEntityAttribute");

            // convert json to list of POJO
            if (StringUtils.isEmpty(dbData)) {
                return null;
            }
            // 转换反斜杠，防止 '\C', '\中' 等无法正常解析
            String originDbData = dbData;
            int i = 0;
            while (i < dbData.length()) {
                // 遍历到反斜杠
                if (dbData.charAt(i) == '\\') {
                    // 不能以反斜杠结尾
                    if (i >= dbData.length() - 1) {
                        throw new IllegalArgumentException("Illegal JSON string(ends with \\): " + originDbData);
                    }
                    // 三种情况下需在当前反斜杠之后添加一个反斜杠
                    // 1. 反斜杠之后并非双引号，如 "key1" : "\a"
                    // 2. 反斜杠之后是双引号，并且是当前键值对的结尾，之后还有其他键值对，如 "key1" : "\", "key2" : "value2"
                    // 3. 反斜杠之后是双引号，并且是当前 json 的结尾，之后没有其他键值对，如 "key1" : "\" }
                    if (dbData.substring(i + 1).matches("^[^\"]+?[\\s\\S]+?$|^\"[\\s]*?,[\\s]*?\"[\\s\\S]+?$|^\"[\\s]*?}[\\s]*?$")) {
                        dbData = dbData.substring(0, i + 1) + "\\" + dbData.substring(i + 1);
                        i++;
                    }
                }
                i++;

            }
            jsonObject = objectMapper.readValue(dbData, JSONObject.class);
            logger.debug("JsonDocumentsConverter.convertToDatabaseColumn" + jsonObject);

        } catch (IOException ex) {
            logger.error(ex.getMessage());
        }
        return jsonObject;
    }
}