package com.uas.ps.sync.producer.jms;

import com.uas.ps.sync.entity.SyncMessage;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.JpaSpecificationExecutor;
import org.springframework.data.jpa.repository.Modifying;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.jpa.repository.query.Procedure;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

/**
 * @author sunyj
 * @since 2018/1/14 16:39
 */
@Repository
public interface SyncMessageDao extends JpaSpecificationExecutor<SyncMessage>, JpaRepository<SyncMessage, Long> {

    /**
     * 获取指定数目的数据
     *
     * @param start 开始的记录
     * @param size  指定数目
     * @return 数据
     */
    @Query(value = "select * from sync$message where me_retry_count < 5 order by me_priority desc, me_id limit ?1, ?2", nativeQuery = true)
    List<SyncMessage> findList(Integer start, Integer size);

    /**
     * 统计行数
     *
     * @return 行数
     */
    @Query(value = "select count(1) from sync$message where me_retry_count < 5", nativeQuery = true)
    long count();

    /**
     * 消息的尝试次数加 1
     */
    @Transactional
    @Modifying
    @Query(value = "update sync$message set me_retry_count = me_retry_count + 1 where me_id = ?1", nativeQuery = true)
    void markRetryCount(Long id);

    /**
     * 出队消息
     *
     * @param id 消息 id
     */
    @Transactional
    @Procedure(procedureName = "sync$dequeue_message")
    void dequeueMessage(Long id);
}
