package com.uas.ps.sync.producer.jms;

import com.uas.ps.sync.entity.SyncMessage;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @author sunyj
 * @since 2018/1/14 16:43
 */
@Service
public class SyncMessageServiceImpl implements SyncMessageService {

    /**
     * 默认的页码
     */
    private static final int PAGE_INDEX = 1;

    /**
     * 默认每页的大小
     */
    private static final int PAGE_SIZE = 20;

    @Autowired
    private SyncMessageDao syncMessageDao;

    @Override
    public SPage<SyncMessage> findAll(Integer page, Integer size) {
        // 处理分页信息
        if (page == null || page <= 0) {
            page = PAGE_INDEX;
        }
        if (size == null || size <= 0) {
            size = PAGE_SIZE;
        }

        SPage<SyncMessage> sPage = new SPage<>();
        long totalElement = syncMessageDao.count();
        sPage.setTotalElement(totalElement);
        // 总数目为0，返回
        if (totalElement == 0) {
            return sPage;
        }

        int totalPage = (int) Math.ceil(totalElement / (1.0 * size));
        sPage.setTotalPage(totalPage);
        // 如果页码过大
        if (page > totalPage) {
            page = totalPage;
        }
        int start = (page - 1) * size;
        sPage.setPage(page);
        sPage.setSize(size);
        if (page == 1) {
            sPage.setFirst(true);
        }
        if (page == totalPage) {
            sPage.setLast(true);
        }
        List<SyncMessage> messages = syncMessageDao.findList(start, size);
        sPage.setContent(messages);
        return sPage;
    }
}
